"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var querystring = require("query-string");

var _require = require("events"),
    EventEmitter = _require.EventEmitter;

var postLog = require("./postLog");
var constants = require("./constants");

var queryParams = querystring.parse(window.location.search);

var TriggerEvents = function (_EventEmitter) {
  _inherits(TriggerEvents, _EventEmitter);

  function TriggerEvents() {
    _classCallCheck(this, TriggerEvents);

    var _this = _possibleConstructorReturn(this, (TriggerEvents.__proto__ || Object.getPrototypeOf(TriggerEvents)).call(this));

    _this.initApp = _this.initApp.bind(_this);
    _this.sendAppStatusPreloaded = _this.sendAppStatusPreloaded.bind(_this);
    _this.sendAppStatusStarted = _this.sendAppStatusStarted.bind(_this);
    _this.sendAppStatusFinished = _this.sendAppStatusFinished.bind(_this);
    _this.sendAppError = _this.sendAppError.bind(_this);
    _this.callbackSuccess = _this.callbackSuccess.bind(_this);
    _this.callbackError = _this.callbackError.bind(_this);
    return _this;
  }

  _createClass(TriggerEvents, [{
    key: "initApp",
    value: function initApp(metainfo) {
      var app = metainfo.app,
          auth = metainfo.auth,
          stage = metainfo.stage;

      if (app && auth && stage) {
        var id = app.id,
            name = app.name,
            version = app.version;
        var organizationId = auth.organizationId,
            teamId = auth.teamId,
            screenId = auth.screenId;

        if (id && name && version && organizationId && teamId && screenId) {
          this.app = {
            id: id,
            name: name,
            version: version
          };
          this.auth = {
            organizationId: organizationId,
            teamId: teamId,
            screenId: screenId
          };
          this.stage = stage;
          addListeners.call(this);
          postLog(constants.APP_CONNECT_SUCCEDED, {}, this.app, this.auth, this.stage);
        } else {
          console.error("App initalization failed! 'id' 'name' 'version' 'organizationId' 'teamId' 'screenId' keys required in initApp() argument object.");
        }
      } else {
        console.error("App initalization failed! 'app' 'auth' 'stage' keys required in initApp() argument object.");
      }
    }
  }, {
    key: "sendAppStatusPreloaded",
    value: function sendAppStatusPreloaded() {
      postLog(constants.APP_LOAD_SUCCEEDED, {}, this.app, this.auth, this.stage);
      sendEvent.call(this, constants.APP_EVENT_PRELOADED);
    }
  }, {
    key: "sendAppStatusStarted",
    value: function sendAppStatusStarted() {
      postLog(constants.APP_PLAY_SUCCEEDED, {}, this.app, this.auth, this.stage);
      sendEvent.call(this, constants.APP_EVENT_STARTED);
    }
  }, {
    key: "sendAppStatusFinished",
    value: function sendAppStatusFinished() {
      postLog(constants.APP_STOP_SUCCEEDED, {}, this.app, this.auth, this.stage);
      sendEvent.call(this, constants.APP_EVENT_FINISHED);
    }
  }, {
    key: "sendAppError",
    value: function sendAppError(error) {
      postLog(constants.APP_ERROR, error, this.app, this.auth, this.stage);
      sendError.call(this, "fatal", error);
    }
  }, {
    key: "callbackSuccess",
    value: function callbackSuccess(data) {
      postLog(constants.USERDATA_CALLBACK, data, this.app, this.auth, this.stage);
      sendEvent.call(this, constants.APP_CALLBACK_SUCCESS, data);
    }
  }, {
    key: "callbackError",
    value: function callbackError(error) {
      postLog(constants.APP_ERROR, error, this.app, this.auth, this.stage);
      sendError.call(this, "fatal", error);
    }
  }]);

  return TriggerEvents;
}(EventEmitter);

function sendEvent(type, payload) {
  var eventData = null;
  if (payload) {
    eventData = stringifyJSON({
      session: queryParams.frame_id,
      type: "success",
      payload: payload
    });
  } else {
    eventData = stringifyJSON({
      session: queryParams.frame_id,
      type: "event",
      event: {
        name: type
      }
    });
  }
  this.targetWindow.postMessage(eventData, "*");
}

function sendError(level) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var eventData = stringifyJSON({
    session: queryParams.frame_id,
    type: "error",
    error: {
      level: level,
      message: data
    }
  });
  this.targetWindow.postMessage(eventData, "*");
}

function addListeners() {
  this.targetWindow = window.opener || window.parent;
  if (!window._started) {
    window._started = true;
    window.addEventListener("message", eventListeners.bind(this));
  }
}

function eventListeners(event) {
  var msgJSON = parseJSON(event.data);
  if (msgJSON.type === "command" && msgJSON.command && msgJSON.command.name) {
    switch (msgJSON.command.name) {
      case constants.APP_INCOMING_COMMAND_START:
        /**
         * Temporary solution to prevent logging when player spamming app
         */
        // postLog(constants.APP_PLAY, {}, this.app, this.auth, this.stage);
        this.emit("start");
        break;

      case constants.APP_INCOMING_COMMAND_FINISH:
        /**
         * Temporary solution to prevent logging when player spamming app
         */
        // postLog(constants.APP_STOP, {}, this.app, this.auth, this.stage);
        this.emit("finish");
        break;

      default:
        break;
    }
  }
}

function parseJSON(string) {
  try {
    return JSON.parse(string);
  } catch (e) {
    return {};
  }
}

function stringifyJSON(object) {
  try {
    return JSON.stringify(object);
  } catch (e) {
    return null;
  }
}

module.exports = TriggerEvents;