"use strict";

module.exports = {
  cached: function cached(key, defaultValue, maxAge, purgeExpired) {
    try {
      var data = window.localStorage[key];
      if (data) {
        var entry = JSON.parse(data);
        var now = new Date().getTime();
        if (maxAge === 0 || maxAge && entry.ts < now - maxAge) {
          throw new Error(":: cached data expired key = " + key);
        } else {
          // console.log("CACHED Hit !!");
          return entry.value;
        }
      }
    } catch (e) {
      // console.warn(':: cached error', e)
    }
    return defaultValue;
  },
  cache: function cache(key, value) {
    // console.log(':: cache[' + key + '] = ', value)
    try {
      if (value) {
        window.localStorage[key] = JSON.stringify({
          value: value,
          ts: new Date().getTime()
        });
        // Store the latest success cache in Backup too !!, Back up has no expired time
        // obj.cacheBackup(key, value)
      }
    } catch (e) {
      console.warn(":: cache error", e);
      obj.clearOldBackup();
    }

    return null; // default value
  },


  // a backup data for the case that we don't have any data to show either from above cache expired and no internet to fetch the new one

  cachedBackup: function cachedBackup(key) {
    try {
      // pull with non checking the timestamp
      key = key; // + '_backup'
      var data = window.localStorage[key];
      if (data) {
        // console.log("cachedBackup !!!! CACHED Hit !!");
        var entry = JSON.parse(data);
        return entry.value;
      }
    } catch (e) {
      console.warn(":: no cachedBackup error", e);
    }

    return null;
  },
  clearOldBackup: function clearOldBackup() {
    // console.log("size BEFORE clear old cache --> ");
    obj.checkLocalStorageSize();
    // clear cache that older than 3 days
    var maxAge = 3 * 24 * 60 * 60 * 1000; // 3 days

    for (var i = 0; i < window.localStorage.length; i++) {
      // do something with localStorage.getItem(localStorage.key(i));
      var key = window.localStorage.key(i);
      var data = window.localStorage[key];
      if (data) {
        var entry = JSON.parse(data);

        var now = new Date().getTime();
        if (maxAge && entry.ts < now - maxAge) {
          delete window.localStorage[key];
          // console.log("clear: " + key);
        }
      }
    }

    // console.log("size AFTER clearOldBackup --> ");
    obj.checkLocalStorageSize();
  },
  checkLocalStorageSize: function checkLocalStorageSize() {
    var _lsTotal = 0,
        _xLen,
        _x;
    for (_x in localStorage) {
      _xLen = (localStorage[_x].length + _x.length) * 2;
      _lsTotal += _xLen;
      // console.log(_x.substr(0, 50) + " = " + (_xLen / 1024).toFixed(2) + " KB");
    }
    // console.log("Total = " + (_lsTotal / 1024).toFixed(2) + " KB");
  }
};