"use strict";

var axios = require("axios");
var cacheManager = require("./cacheManager.js");

module.exports = {
  fetchData: function fetchData(dataURL, params, cacheDurationMins, cacheVerifyFn) {
    return new Promise(function (resolve, reject) {
      if (params && typeof params !== "undefined") {
        var cacheKey = escape(dataURL + JSON.stringify(params));
      } else {
        var cacheKey = escape(dataURL);
      }

      // if not specific cacheDurationMins, set default to 30 mins
      if (typeof cacheDurationMins === "undefined" || cacheDurationMins === null) {
        cacheDurationMins = 30;
      }

      // try get data from cache within the specific cache period
      var cachedResult = cacheManager.cached(cacheKey, null, cacheDurationMins * 60 * 1000);

      if (cacheDurationMins !== 0 && cachedResult !== null) {
        // got the cached data, Yay!!
        resolve(cachedResult);
      } else {
        // console.log("fetch dataURL = " + dataURL, params);
        // no cache, let's fetch a fresh one
        axios.get(dataURL, {
          params: params
        }).then(function (response) {
          var result = response.data;

          if (result) {
            if (cacheVerifyFn !== null && cacheVerifyFn) {
              if (cacheVerifyFn(result)) {
                // console.log('Allow cache PASS verification')
                cacheManager.cache(cacheKey, result);
              } else {
                // console.log("Invalid response data, NO SAVE the cache !!");
                // return previous cached data instead
                return resolve(cacheManager.cachedBackup(cacheKey));
              }
            } else {
              // no verify function = then auto save the results
              cacheManager.cache(cacheKey, result);
            }
            // console.log("DataLoader: return with result: ", result);
            return resolve(result);
          } else {
            // try to grab data from backup, result from backup can be 'null'
            var cachedResult = cacheManager.cachedBackup(cacheKey);
            resolve(cachedResult);
          }
        }).catch(function (response) {
          console.warn("fetch failed with error = ", response);
          // try to grab data from backup, result from backup can be 'null'
          var cachedResult = cacheManager.cachedBackup(cacheKey);
          if ((cachedResult === null || !cachedResult) && response.data && response.data.response && response.data.response.text) {
            return resolve({ error: response.data.response.text });
          }
          return resolve(cachedResult);
        });
      }
    });
  }
};